unit Main;

interface

uses
  Windows, OpenGL12, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls;

type
  TMainForm = class(TForm)
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    FDC: HDC;
    FRC: HGLRC;
    FAngle: Integer;
    FMainList: Cardinal;
    FNeedSetup: Boolean;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure Setup;
  protected
    procedure CreateWnd; override;
  end;

var
  MainForm: TMainForm;

//----------------------------------------------------------------------------------------------------------------------

implementation

{$R *.DFM}

//----------------------------------------------------------------------------------------------------------------------

function CheckRes: Boolean;

var
  Error: TGLEnum;
  
begin
  Error := glGetError;
  Result := Error = GL_NO_ERROR;
  if not Result then
    ShowMessage('Fehler: ' + gluErrorString(Error));
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TMainForm.WMEraseBkgnd(var Message: TWMEraseBkgnd);

begin
  Message.Result := 1;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TMainForm.FormCreate(Sender: TObject);

begin
  FNeedSetup := True;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TMainForm.FormDestroy(Sender: TObject);

begin
  ActivateRenderingContext(FDC, FRC);
  glDeleteLists(FMainList, 1);
  DeactivateRenderingContext;
  DestroyRenderingContext(FRC);
  ReleaseDC(Handle, FDC);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TMainForm.FormPaint(Sender: TObject);

begin
  if FNeedSetup then
    Setup;
    
  ActivateRenderingContext(FDC, FRC);

  // clear color and depth buffer
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  // activate transformation matrix
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity;
  // viewer position and direction
  gluLookAt(0, 0, 6, 0, 0, -10, 0, 1, 0);

  // ein wenig Animation
  glRotatef(FAngle, 1, 1, 0);
  glRotatef(2 * FAngle, 0, 1, 1);
  glCallList(FMainList);
  SwapBuffers(FDC);
  DeactivateRenderingContext;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TMainForm.Timer1Timer(Sender: TObject);

begin
  FAngle := (FAngle + 1) mod 360;
  Invalidate;
end;

//----------------------------------------------------------------------------------------------------------------------

function BuildObject: Cardinal;

// Separate function to build a scene.

var
  Quadric: PGLUquadricObj;

begin
  Result := glGenLists(1);
  glNewList(Result, GL_COMPILE);
    Quadric := gluNewQuadric;
    gluQuadricDrawStyle(Quadric, GLU_FILL);
    gluQuadricNormals(Quadric, GLU_SMOOTH);
    gluQuadricOrientation(Quadric, GLU_OUTSIDE);
    glRotatef(-90, 1, 0, 0);
    glTranslated(0, 0, -0.5);
    gluCylinder(Quadric, 1, 0.5, 1, 50, 50);
    glPushMatrix;
    glTranslatef(0, 0, 1);
    gluDisk(Quadric, 0, 0.5, 50, 50);
    glPopMatrix;
    gluQuadricOrientation(Quadric, GLU_INSIDE);
    gluDisk(Quadric, 0, 1, 50, 50);
    gluDeleteQuadric(Quadric);
  glEndList;
  if not CheckRes then Halt;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TMainForm.Setup;

const
  Light1: array[0..3] of Single = (-30, 30, 30, 1);
  Light2: array[0..3] of Single = (50, 100, 50, 1);

  Ambient: array[0..3] of Single = (0.2, 0.2, 0.2, 1);
  Diffuse1: array[0..3] of Single = (1, 1, 1, 1);
  Diffuse2: array[0..3] of Single = (0, 0.7, 0.37, 1);
  Diffuse3: array[0..3] of Single = (0.5, 0.5, 0.5, 1);
  Specular: array[0..3] of Single = (1, 1, 1, 1);

var
  Dummy: HPalette;
  
begin
  FNeedSetup := False;
  FRC := CreateRenderingContext(FDC, [opDoubleBuffered], 32, 0, 0, 0, 0, Dummy);
  ActivateRenderingContext(FDC, FRC);

  glEnable(GL_DEPTH_TEST);
  glEnable(GL_CULL_FACE);

  // set up 2 light sources
  glEnable(GL_LIGHTING);
  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, Ord(GL_FALSE));

  glEnable(GL_LIGHT0);
  glLightfv(GL_LIGHT0, GL_POSITION, @Light1);
  glLightfv(GL_LIGHT0, GL_AMBIENT, @Ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, @Diffuse1);
  glLightfv(GL_LIGHT0, GL_SPECULAR, @Specular);

  glEnable(GL_LIGHT1);
  glLightfv(GL_LIGHT1, GL_POSITION, @Light2);
  glLightfv(GL_LIGHT1, GL_AMBIENT, @Ambient);
  glLightfv(GL_LIGHT1, GL_DIFFUSE, @Diffuse2);
  glLightfv(GL_LIGHT1, GL_SPECULAR, @Specular);

  // create a display list for the actual scene (here a cylinder)
  FMainList := BuildObject;

  glMateriali(GL_FRONT, GL_SHININESS, 128);
  glMaterialfv(GL_FRONT, GL_AMBIENT, @Ambient);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, @Diffuse3);
  glMaterialfv(GL_FRONT, GL_SPECULAR, @Specular);

  // background color
  glClearColor(0.25, 0.25, 0.25, 1);

  DeactivateRenderingContext;

  // Important to setup the viewport!
  FormResize(Self);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TMainForm.FormResize(Sender: TObject);

begin
  if HandleAllocated and not FNeedSetup then
  begin
    ActivateRenderingContext(FDC, FRC);
    glViewport(0, 0, Width, Height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity;
    gluPerspective(35, Width / Height, 1, 100);
    DeactivateRenderingContext;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TMainForm.CreateWnd;

begin
  inherited;
  FDC := GetDC(Handle);
end;

//----------------------------------------------------------------------------------------------------------------------

end.


