unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  GLControl, ExtCtrls, StdCtrls;

type
  TMainForm = class(TForm)
    GLControl1: TGLControl;
    GLControl2: TGLControl;
    GLControl3: TGLControl;
    GLControl4: TGLControl;
    Timer1: TTimer;
    procedure GLControlRender(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure GLControlSetup(Sender: TObject);
  private
    FAngle: Integer;
    FNeedSetup: Boolean;
    FList: Cardinal;
    FFirstContext: HGLRC;
    procedure Setup;
  end;

var
  MainForm: TMainForm;

//----------------------------------------------------------------------------------------------------------------------

implementation

uses
  OpenGL12;
  
{$R *.DFM}

//----------------------------------------------------------------------------------------------------------------------

procedure TMainForm.Setup;

begin
  FNeedSetup := False;
  FList := glGenLists(1);
  glNewList(FList, GL_COMPILE);
  glEnable(GL_DEPTH_TEST);
  glFrontFace(GL_CCW);
  glBegin(GL_TRIANGLE_FAN); // a color tetrahedron
    glColor3f(1, 1, 1);       // color of the upper vertex
    glVertex3f(0, 1, 0);      // coordinates of the upper vertex (= center of the fan)
    glColor3f(1, 0, 0);       // color left-front vertex
    glVertex3f(-1, 0, 1);     // coordinates of the left-front vertex
    glColor3f(0, 1, 0);       // right-front
    glVertex3f(1, 0, 1);      // right-front
    glColor3f(0, 0, 1);       // middle-back
    glVertex3f(0, 0, -1);     // middle-back
    glColor3f(1, 0, 0);       // left-front again (close fan)
    glVertex3f(-1, 0, 1);     // left-front
  glEnd;                    // finish triangle fan
  glEndList;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TMainForm.GLControlRender(Sender: TObject);

begin
  with Sender as TGLControl do
  begin
    gluLookAt(0, 0, 6, 0, 0, -10, 0, 1, 0);

    glRotatef(FAngle, 1, 1, 1);
    case Tag of
      0:
        glRotatef(30, 1, 0, 0);
      1:
        glRotatef(0, 1, 0, 0);
      2:
        glRotatef(15, 1, 0, 0);
      3:
        glRotatef(80, 1, 0, 0);
    end;
    glCallList(FList);
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TMainForm.Timer1Timer(Sender: TObject);

begin
  FAngle := (FAngle + 5) mod 360;
  GLControl1.Invalidate;
  GLControl2.Invalidate;
  GLControl3.Invalidate;
  GLControl4.Invalidate;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TMainForm.FormCreate(Sender: TObject);

begin
  FNeedSetup := True;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TMainForm.GLControlSetup(Sender: TObject);

begin
  // Create the base display list if not yet done.
  if FNeedSetup then
  begin
    Setup;
    // The current rendering context will become the source for context sharing.
    FFirstContext := TGLControl(Sender).RenderingContext;
  end
  else
    // One for all. The display list will be shared between all control instances.
    wglShareLists(FFirstContext, TGLControl(Sender).RenderingContext);

end;

//----------------------------------------------------------------------------------------------------------------------

end.

